clear all
close all

%% README
% Two layers migration. 
% 
% Contents:
%   TwoLayersMigration.m, this script
%   plot_model.m, script for plotting some figures
%   cal_time_angle_slw.m, function for calculating the traveltime and the angle of the ray vector of the Ps wave
%   ricker.m, function that defining the ricker wavelet

% Instruction:
%   Run plot_model.m first to check how these parameters change the model. Then play with this script.
%


%% Initialization

% model parameters
model.moho_depth   = 30.0;  % interface depth in km
model.moho_dipping = 30.0;  % dipping angle of the interface
model.Vp_crust     = 6.0;   % first layer, Vp in km/s
model.Vs_crust     = 3.42;  % first layer, Vs
model.Vp_mantle    = 8.0;   % second layer, Vp
model.Vs_mantle    = 4.45;  % second layer, Vs

% stations
station_spacing = 10;
x_station = 10:station_spacing:290;

% slowness range
slw_range = [-8:-4 4:8];
slowness  = 0.01 * slw_range;   % slowness range of teleseismic events (0.04 ~ 0.08 s/km)

% width of the wavelet
gauss_or_ricker = 1;    % 1 for gauss and others for ricker

if (gauss_or_ricker == 1)
    width_gauss = 0.05;
else
    freq_ricker = 1.0;
end

% parameters of migration area
dist_vec  = 0:1:300;
depth_vec = 0:1:200;

potential = zeros(length(depth_vec), length(dist_vec));


%
moho        = model.moho_depth;
phi_dipping = model.moho_dipping;
Vp_crust    = model.Vp_crust;
Vs_crust    = model.Vs_crust;
Vp_mantle   = model.Vp_mantle;
Vs_mantle   = model.Vs_mantle;



%% Forward modeling 

for islw = 1:length(slw_range)
    
    slowness_loop = slowness(islw);    % slowness of plane wave in second layer
    theta_P = asind(slowness_loop*Vp_mantle);    % incidence of P wave in second layer


    % traveltime of direct P wave (DP)
    alpha_DP = asind(sind(phi_dipping - theta_P)*Vp_crust/Vp_mantle);
    x_inter_DP = (x_station - moho*tand(phi_dipping - alpha_DP))/(1 + tand(phi_dipping)*tand(phi_dipping - alpha_DP));
    d_inter_DP = x_inter_DP * tand(phi_dipping) + moho;

    time_p1_DP = -(x_inter_DP * sind(phi_dipping - theta_P))/(Vp_mantle * cosd(phi_dipping));
    time_p2_DP = sqrt((x_station - x_inter_DP).^2 + (d_inter_DP).^2)/Vp_crust;

    traveltime_DP = time_p1_DP + time_p2_DP;

    % traveltime of Ps wave (Ps)
    beta_Ps = asind(sind(phi_dipping - theta_P)*Vs_crust/Vp_mantle);
    x_inter_Ps = (x_station - moho*tand(phi_dipping - beta_Ps))/(1 + tand(phi_dipping)*tand(phi_dipping - beta_Ps));
    d_inter_Ps = x_inter_Ps * tand(phi_dipping) + moho;

    time_p1_Ps = -(x_inter_Ps * sind(phi_dipping - theta_P))/(Vp_mantle * cosd(phi_dipping));
    time_s2_Ps = sqrt((x_station - x_inter_Ps).^2 + (d_inter_Ps).^2)/Vs_crust;

    sign_Ps = sign( slowness_loop * sind( 2*(phi_dipping - beta_Ps - theta_P + 90.0) ) );   % sign of the Ps conversion
    traveltime_Ps = time_p1_Ps + time_s2_Ps;

    % traveltime difference between Ps and direct P wave

    diff_time = traveltime_Ps - traveltime_DP;


    %% 1D migration tests

    theta_S_Ps = asind(slowness_loop*Vs_crust);

    for ista = 1:length(diff_time)
        for idep = 1:length(depth_vec)
            depth_loop = depth_vec(idep);
            for idist = 1:length(dist_vec)
                dist_loop = dist_vec(idist);

           
%                 % cutoff parameters (horizontal distance from theoritical raypath with flat layers)              
%                 if ( abs(dist_loop - (x_station(ista) - depth_loop * tand(theta_S_Ps)) ) > 2 )                
%                     continue;
%                 end


%                 % cutoff parameters (angle from theoritical raypath)
%                 theta_tmp = atand((x_station(ista) - dist_loop)/(depth_loop));
%                 diff_angle = theta_tmp - theta_S_Ps;
%                 if ( abs(diff_angle) > 5 )
%                     continue;
%                 end

                [traveltime_loop, angle_scatter, sign_scatter, incidence_station] = cal_time_angle_slw(model, dist_loop, depth_loop, x_station(ista), slowness_loop);
                if (traveltime_loop == -1)
                    continue;
                end

                difftime_loop = traveltime_loop - traveltime_DP(ista);
                
                scatter_coefficient = sind(2*angle_scatter);
                station_correction  = sign_scatter * cosd(incidence_station);
                
                if (gauss_or_ricker == 1)
                    potential(idep,idist) = potential(idep, idist) + station_correction * scatter_coefficient * ( sign_Ps * gaussmf(difftime_loop,[width_gauss diff_time(ista)]) );
                else
                    potential(idep,idist) = potential(idep, idist) + station_correction * scatter_coefficient * ( sign_Ps * ricker(difftime_loop-diff_time(ista),freq_ricker) );
                end

            end
        end
    end
end


%% plotting

figure(4)
normalized_fac = max(max(potential));
%normalized_fac = 1.0;
imagesc(dist_vec,depth_vec,potential./normalized_fac)
colorbar
width_height_ratio = dist_vec(end)/depth_vec(end);
set(gcf,'Units','centimeters','InnerPosition',[5 5 15*[width_height_ratio 1]])
caxis([-1 1])
xlabel('distance(km)')
ylabel('depth(km)')

figure(5)
imagesc(dist_vec,depth_vec,potential./normalized_fac)
colorbar
hold on
plot(dist_vec,tand(phi_dipping).*dist_vec+moho,'r--')
%plot(x_station(ista)-depth_vec*tand(asind(slowness*Vs_crust)),depth_vec,'g')
set(gcf,'Units','centimeters','InnerPosition',[5 5 15*[width_height_ratio 1]])
caxis([-1 1])
xlabel('distance(km)')
ylabel('depth(km)')
hold off

% print(['mig_result_no_ref_'   num2str(phi_dipping) '_slw_0.04_0.08.jpeg'],figure(4),'-djpeg','-r900')
% print(['mig_result_with_ref_' num2str(phi_dipping) '_slw_0.04_0.08.jpeg'],figure(5),'-djpeg','-r900')
% print(['mig_result_no_ref_slw_0.04_0.12.jpeg'],figure(4),'-djpeg','-r900')
% print(['mig_result_with_ref_slw_0.04_0.12.jpeg'],figure(5),'-djpeg','-r900')
% print(['mig_result_no_ref_slw_0.09_0.12.jpeg'],figure(4),'-djpeg','-r900')
% print(['mig_result_with_ref_slw_0.09_0.12.jpeg'],figure(5),'-djpeg','-r900')
