clear all
close all

%% Forward modeling 

% model parameters
model.moho_depth   = 30.0;  % interface depth in km
model.moho_dipping = 30.0;  % dipping angle of the interface
model.Vp_crust     = 6.0;   % first layer, Vp in km/s
model.Vs_crust     = 3.42;  % first layer, Vs
model.Vp_mantle    = 8.0;   % second layer, Vp
model.Vs_mantle    = 4.45;  % second layer, Vs

% input parameters
slowness = 0.01;    % slowness of plane wave in second layer, positive means from left to right, do not use zero
    
% stations
station_spacing = 5;
x_station = 10:station_spacing:290;

% width of gauss wavelet
width_gauss = 0.1;


% initialization
moho        = model.moho_depth;
phi_dipping = model.moho_dipping;
Vp_crust    = model.Vp_crust;
Vs_crust    = model.Vs_crust;
Vp_mantle   = model.Vp_mantle;
Vs_mantle   = model.Vs_mantle;

% angles 
theta_P = asind(slowness*Vp_mantle);    % incidence of P wave in second layer

% traveltime of direct P wave (DP)
alpha_DP = asind(sind(phi_dipping - theta_P)*Vp_crust/Vp_mantle);
x_inter_DP = (x_station - moho*tand(phi_dipping - alpha_DP))/(1 + tand(phi_dipping)*tand(phi_dipping - alpha_DP));
d_inter_DP = x_inter_DP * tand(phi_dipping) + moho;

time_p1_DP = -(x_inter_DP * sind(phi_dipping - theta_P))/(Vp_mantle * cosd(phi_dipping));
time_p2_DP = sqrt((x_station - x_inter_DP).^2 + (d_inter_DP).^2)/Vp_crust;

traveltime_DP = time_p1_DP + time_p2_DP;

% traveltime of Ps wave (Ps)
beta_Ps = asind(sind(phi_dipping - theta_P)*Vs_crust/Vp_mantle);
x_inter_Ps = (x_station - moho*tand(phi_dipping - beta_Ps))/(1 + tand(phi_dipping)*tand(phi_dipping - beta_Ps));
d_inter_Ps = x_inter_Ps * tand(phi_dipping) + moho;

time_p1_Ps = -(x_inter_Ps * sind(phi_dipping - theta_P))/(Vp_mantle * cosd(phi_dipping));
time_s2_Ps = sqrt((x_station - x_inter_Ps).^2 + (d_inter_Ps).^2)/Vs_crust;
    
sign_Ps = sign( slowness * sind( 2*(phi_dipping - beta_Ps - theta_P + 90.0) ) );   % sign of the Ps conversion
traveltime_Ps = time_p1_Ps + time_s2_Ps;

% traveltime difference between Ps and direct P wave

diff_time = traveltime_Ps - traveltime_DP;


%% plot model setting
max_dist  = x_station(length(x_station)) + moho;
max_depth = x_station(length(x_station)) * tand(phi_dipping) + 2 * moho;
clf,figure(1)
% moho(interface) 
plot([0, max_dist],[moho, max_dist*tand(phi_dipping) + moho],'g')
hold on
% reference line at moho depth
plot([0, moho],[moho, moho],'--k')
% surface
plot([0, max_dist],[0, 0],'k')
% stations
scatter(x_station,zeros(length(x_station),1)-1,30,'k','filled','v')

% raypath of the direct P wave (DP)
c_index = round(0.5*length(x_station));
plot([x_inter_DP(c_index), x_station(c_index)], [d_inter_DP(c_index), 0],'b')
c_dist_DP = x_inter_DP(c_index) - (max_depth - d_inter_DP(c_index)) * tand(theta_P);
plot([c_dist_DP, x_inter_DP(c_index)], [max_depth, d_inter_DP(c_index)], 'b')
% raypath of the Ps wave (Ps)
plot([x_inter_Ps(c_index), x_station(c_index)], [d_inter_Ps(c_index), 0],'r')
c_dist_Ps = x_inter_Ps(c_index) - (max_depth - d_inter_Ps(c_index)) * tand(theta_P);
plot([c_dist_Ps, x_inter_Ps(c_index)], [max_depth, d_inter_Ps(c_index)], 'b')

% angles illustration for alpha and beta
height_for_normal = 25;
dist_above_DP  = x_inter_DP(c_index) + height_for_normal*tand(phi_dipping);
dist_below_DP  = x_inter_DP(c_index) - height_for_normal*tand(phi_dipping);
plot([dist_above_DP, dist_below_DP],[d_inter_DP(c_index)-height_for_normal, d_inter_DP(c_index)+height_for_normal],'--k')
% text(dist_above_DP - 0.1 * height_for_normal*tand(phi_dipping), d_inter_DP(c_index)-0.84*height_for_normal,'\leftarrow\alpha','HorizontalAlignment','center')

dist_above_Ps  = x_inter_Ps(c_index) + height_for_normal*tand(phi_dipping);
dist_below_Ps  = x_inter_Ps(c_index) - height_for_normal*tand(phi_dipping);
plot([dist_above_Ps, dist_below_Ps],[d_inter_Ps(c_index)-height_for_normal, d_inter_Ps(c_index)+height_for_normal],'--k')
% text(dist_above_Ps - 0.6 * height_for_normal*tand(phi_dipping), d_inter_DP(c_index)-height_for_normal,'\beta\rightarrow','HorizontalAlignment','center')

xlim([0 max_dist])
ylim([-5 max_depth])
set(gca,'Ydir','reverse')
width_height_ratio = max_dist/max_depth;
set(gcf,'Units','centimeters','InnerPosition',[5 5 15*[width_height_ratio 1]])
xlabel('distance(km)')
ylabel('depth(km)')

text(0.7*moho, 0.7*moho*tand(0.5*phi_dipping)+moho, '\leftarrow dipping angle')
text(0.7*max_dist, 0.7*max_dist*tand(phi_dipping) + moho - 5, '\downarrow moho(interface)')


textstr = {'first layer' ;['Vp=' num2str(Vp_crust) 'km/s'] ;['Vs=' num2str(Vs_crust) 'km/s']};
text(max_dist-60, 20, textstr)
textstr = {'second layer';['Vp=' num2str(Vp_mantle) 'km/s'];['Vs=' num2str(Vs_mantle) 'km/s']};
text(10, 80, textstr)

%print('model_setting.jpeg',figure(1),'-djpeg','-r900')


%% plot synthetic structure response

max_time = 10*round(traveltime_Ps(end)/10) + 10;
time_vec = [0:0.01:50];
amp_factor = 3.0;
for ii = 1:length(x_station)
    record(:,ii) = x_station(ii) + amp_factor * (gaussmf(time_vec, [width_gauss traveltime_DP(ii)]) + sign_Ps * gaussmf(time_vec, [width_gauss traveltime_Ps(ii)]));
end

figure(2)
hold on
for ii = 1:length(x_station)
    plot(time_vec,record(:,ii),'k')
end
xlabel('absolute time(s)')
ylabel('distance(km)')


max_diff_time = 10*round(diff_time(end)/10) + 10;
diff_time_vec = [-1:0.01:max_diff_time];
figure(3)
hold on
for ii = 1:length(x_station)
    diff_record(:,ii) = x_station(ii) + amp_factor * (gaussmf(diff_time_vec, [width_gauss 0.0]) + sign_Ps * gaussmf(diff_time_vec, [width_gauss diff_time(ii)]));
end

for ii = 1:length(x_station)
    plot(diff_time_vec,diff_record(:,ii),'k')
end
xlim([-1 diff_time_vec(end)])
xlabel('differential time(s)')
ylabel('distance(km)')

% print('synthetics_abs.jpeg',figure(2),'-djpeg','-r900')
% print('synthetics_rel.jpeg',figure(3),'-djpeg','-r900')


%% 1D migration tests

dist_vec  = 0:1:300;
depth_vec = 0:1:200;

potential = zeros(length(depth_vec), length(dist_vec));

theta_S_Ps = asind(slowness*Vs_crust);

for ista = round(0.5*length(diff_time))
    for idep = 1:length(depth_vec)
        depth_loop = depth_vec(idep);
        for idist = 1:length(dist_vec)
            dist_loop = dist_vec(idist);
            
%             % cutoff parameters (horizontal distance from theoritical raypath with flat layers)
%             if ( abs(dist_loop - (x_station(ista) - depth_loop * tand(theta_S_Ps))) > 30 )
%                 continue;
%             end


%             % cutoff parameters (angle from theoritical raypath)
%             theta_tmp = atand((x_station(ista) - dist_loop)/(depth_loop));
%             diff_angle = theta_tmp - theta_S_Ps;
%             if ( abs(diff_angle) > 20 )
%                 continue;
%             end
            
            [traveltime_loop, angle_scatter, sign_scatter, incidence_station] = cal_time_angle_slw(model, dist_loop, depth_loop, x_station(ista), slowness);
            if (traveltime_loop == -1)
                continue;
            end

            difftime_loop = traveltime_loop - traveltime_DP(ista);

            scatter_coefficient = sign_scatter * sind(2*angle_scatter);
            station_correction  = cosd(incidence_station);

            potential(idep,idist) = potential(idep, idist) + station_correction * scatter_coefficient * (sign_Ps * gaussmf(difftime_loop,[width_gauss diff_time(ista)]));

        end
    end
end

figure(4)
normalized_fac = max(max(potential));
% normalized_fac = 1.0;
imagesc(dist_vec,depth_vec,potential./normalized_fac)
colorbar
width_height_ratio = dist_vec(end)/depth_vec(end);
set(gcf,'Units','centimeters','InnerPosition',[5 5 15*[width_height_ratio 1]])
caxis([-1 1])
xlabel('distance(km)')
ylabel('depth(km)')

figure(5)
imagesc(dist_vec,depth_vec,potential./normalized_fac)
colorbar
hold on
plot(dist_vec,tand(phi_dipping).*dist_vec+moho,'r')
%plot(x_station(ista)-depth_vec*tand(asind(slowness*Vs_crust)),depth_vec,'g')
set(gcf,'Units','centimeters','InnerPosition',[5 5 15*[width_height_ratio 1]])
caxis([-1 1])
xlabel('distance(km)')
ylabel('depth(km)')
hold off
% print(['mig_result_no_ref_'   num2str(phi_dipping) '.jpeg'],figure(4),'-djpeg','-r900')
% print(['mig_result_with_ref_' num2str(phi_dipping) '.jpeg'],figure(5),'-djpeg','-r900')


